@@ -10,7 +10,6 @@
 -- Generally accessible in o(1)
 --
 {-# LANGUAGE MagicHash #-}
-{-# LANGUAGE UnboxedTuples #-}
 module Foundation.Array
     ( Array
     , MArray
@@ -24,7 +23,7 @@ module Foundation.Array
     , OutOfBound
     ) where
 
-import           Foundation.Array.Common
+import           Foundation.Primitive.Exception
 import           Foundation.Array.Boxed
 import           Foundation.Array.Unboxed
 import           Foundation.Array.Unboxed.Mutable
