@@ -16,9 +16,7 @@ module Foundation.Array.Boxed
     , MArray
     , empty
     , length
-    , lengthSize
     , mutableLength
-    , mutableLengthSize
     , copy
     , unsafeCopyAtRO
     , thaw
@@ -54,34 +52,40 @@ module Foundation.Array.Boxed
     , sortBy
     , filter
     , reverse
+    , elem
     , find
     , foldl'
     , foldr
-    , foldl
+    , foldl1'
+    , foldr1
+    , all
+    , any
+    , isPrefixOf
+    , isSuffixOf
     , builderAppend
     , builderBuild
+    , builderBuild_
     ) where
 
 import           GHC.Prim
 import           GHC.Types
 import           GHC.ST
 import           Foundation.Numerical
+import           Foundation.Collection.NonEmpty
 import           Foundation.Internal.Base
 import           Foundation.Internal.Proxy
 import           Foundation.Internal.MonadTrans
 import           Foundation.Primitive.Types.OffsetSize
 import           Foundation.Primitive.Types
 import           Foundation.Primitive.NormalForm
-import           Foundation.Primitive.IntegralConv
 import           Foundation.Primitive.Monad
-import           Foundation.Array.Common
+import           Foundation.Primitive.Exception
 import           Foundation.Boot.Builder
 import qualified Foundation.Boot.List as List
-import qualified Prelude
 
 -- | Array of a
 data Array a = Array {-# UNPACK #-} !(Offset a)
-                     {-# UNPACK #-} !(Size a)
+                     {-# UNPACK #-} !(CountOf a)
                                     (Array# a)
     deriving (Typeable)
 
@@ -96,14 +100,14 @@ arrayType = mkNoRepType "Foundation.Array"
 instance NormalForm a => NormalForm (Array a) where
     toNormalForm arr = loop 0
       where
-        !sz = lengthSize arr
+        !sz = length arr
         loop !i
             | i .==# sz = ()
             | otherwise = unsafeIndex arr i `seq` loop (i+1)
 
 -- | Mutable Array of a
 data MArray a st = MArray {-# UNPACK #-} !(Offset a)
-                          {-# UNPACK #-} !(Size a)
+                          {-# UNPACK #-} !(CountOf a)
                                          (MutableArray# st a)
     deriving (Typeable)
 
@@ -130,11 +134,11 @@ instance IsList (Array ty) where
 
 -- | return the numbers of elements in a mutable array
 mutableLength :: MArray ty st -> Int
-mutableLength (MArray _ (Size len) _) = len
+mutableLength (MArray _ (CountOf len) _) = len
 {-# INLINE mutableLength #-}
 
 -- | return the numbers of elements in a mutable array
-mutableLengthSize :: MArray ty st -> Size ty
+mutableLengthSize :: MArray ty st -> CountOf ty
 mutableLengthSize (MArray _ size _) = size
 {-# INLINE mutableLengthSize #-}
 
@@ -145,7 +149,7 @@ index :: Array ty -> Offset ty -> ty
 index array n
     | isOutOfBound n len = outOfBound OOB_Index n len
     | otherwise          = unsafeIndex array n
-  where len = lengthSize array
+  where len = length array
 {-# INLINE index #-}
 
 -- | Return the element at a specific index from an array without bounds checking.
@@ -215,9 +219,9 @@ unsafeThaw (Array ofs sz a) = primitive $ \st -> (# st, MArray ofs sz (unsafeCoe
 -- and every values is copied, before returning the mutable array.
 thaw :: PrimMonad prim => Array ty -> prim (MArray ty (PrimState prim))
 thaw array = do
-    m <- new (lengthSize array)
-    unsafeCopyAtRO m (Offset 0) array (Offset 0) (lengthSize array)
-    return m
+    m <- new (length array)
+    unsafeCopyAtRO m (Offset 0) array (Offset 0) (length array)
+    pure m
 {-# INLINE thaw #-}
 
 freeze :: PrimMonad prim => MArray ty (PrimState prim) -> prim (Array ty)
@@ -238,7 +242,7 @@ copyAt :: PrimMonad prim
        -> Offset ty                  -- ^ offset at destination
        -> MArray ty (PrimState prim) -- ^ source array
        -> Offset ty                  -- ^ offset at source
-       -> Size ty                    -- ^ number of elements to copy
+       -> CountOf ty                    -- ^ number of elements to copy
        -> prim ()
 copyAt dst od src os n = loop od os
   where -- !endIndex = os `offsetPlusE` n
@@ -256,26 +260,26 @@ unsafeCopyAtRO :: PrimMonad prim
                -> Offset ty                  -- ^ offset at destination
                -> Array ty                   -- ^ source array
                -> Offset ty                  -- ^ offset at source
-               -> Size ty                    -- ^ number of elements to copy
+               -> CountOf ty                    -- ^ number of elements to copy
                -> prim ()
 unsafeCopyAtRO (MArray (Offset (I# dstart)) _ da) (Offset (I# dofs))
                (Array  (Offset (I# sstart)) _ sa) (Offset (I# sofs))
-               (Size (I# n)) =
+               (CountOf (I# n)) =
     primitive $ \st ->
         (# copyArray# sa (sstart +# sofs) da (dstart +# dofs) n st, () #)
 
 -- | Allocate a new array with a fill function that has access to the elements of
 --   the source array.
 unsafeCopyFrom :: Array ty -- ^ Source array
-               -> Size ty  -- ^ Length of the destination array
+               -> CountOf ty  -- ^ Length of the destination array
                -> (Array ty -> Offset ty -> MArray ty s -> ST s ())
                -- ^ Function called for each element in the source array
                -> ST s (Array ty) -- ^ Returns the filled new array
 unsafeCopyFrom v' newLen f = new newLen >>= fill (Offset 0) f >>= unsafeFreeze
-  where len = lengthSize v'
+  where len = length v'
         endIdx = Offset 0 `offsetPlusE` len
         fill i f' r'
-            | i == endIdx = return r'
+            | i == endIdx = pure r'
             | otherwise   = do f' v' i r'
                                fill (i + Offset 1) f' r'
 
@@ -285,14 +289,14 @@ unsafeCopyFrom v' newLen f = new newLen >>= fill (Offset 0) f >>= unsafeFreeze
 --
 -- All mutable arrays are allocated on a 64 bits aligned addresses
 -- and always contains a number of bytes multiples of 64 bits.
-new :: PrimMonad prim => Size ty -> prim (MArray ty (PrimState prim))
-new sz@(Size (I# n)) = primitive $ \s1 ->
+new :: PrimMonad prim => CountOf ty -> prim (MArray ty (PrimState prim))
+new sz@(CountOf (I# n)) = primitive $ \s1 ->
     case newArray# n (error "vector: internal error uninitialized vector") s1 of
         (# s2, ma #) -> (# s2, MArray (Offset 0) sz ma #)
 
 -- | Create a new array of size @n by settings each cells through the
 -- function @f.
-create :: forall ty . Size ty -- ^ the size of the array
+create :: forall ty . CountOf ty -- ^ the size of the array
        -> (Offset ty -> ty)   -- ^ the function that set the value at the index
        -> Array ty            -- ^ the array created
 create n initializer = runST (new n >>= iter initializer)
@@ -310,9 +314,9 @@ create n initializer = runST (new n >>= iter initializer)
 -- higher level collection implementation
 -----------------------------------------------------------------------
 equal :: Eq a => Array a -> Array a -> Bool
-equal a b = (len == lengthSize b) && eachEqual 0
+equal a b = (len == length b) && eachEqual 0
   where
-    len = lengthSize a
+    len = length a
     eachEqual !i
         | i .==# len                         = True
         | unsafeIndex a i /= unsafeIndex b i = False
@@ -321,8 +325,8 @@ equal a b = (len == lengthSize b) && eachEqual 0
 vCompare :: Ord a => Array a -> Array a -> Ordering
 vCompare a b = loop 0
   where
-    !la = lengthSize a
-    !lb = lengthSize b
+    !la = length a
+    !lb = length b
     loop n
         | n .==# la = if la == lb then EQ else LT
         | n .==# lb = GT
@@ -334,16 +338,13 @@ vCompare a b = loop 0
 empty :: Array a
 empty = runST $ onNewArray 0 (\_ s -> s)
 
-length :: Array a -> Int
-length (Array _ (Size len) _) = len
-
-lengthSize :: Array a -> Size a
-lengthSize (Array _ sz _) = sz
+length :: Array a -> CountOf a
+length (Array _ sz _) = sz
 
 vFromList :: [a] -> Array a
 vFromList l = runST (new len >>= loop 0 l)
   where
-    len = Size $ List.length l
+    len = CountOf $ List.length l
     loop _ []     ma = unsafeFreeze ma
     loop i (x:xs) ma = unsafeWrite ma i x >> loop (i+1) xs ma
 
@@ -351,7 +352,7 @@ vToList :: Array a -> [a]
 vToList v
     | len == 0  = []
     | otherwise = fmap (unsafeIndex v) [0..sizeLastOffset len]
-  where !len = lengthSize v
+  where !len = length v
 
 -- | Append 2 arrays together by creating a new bigger array
 append :: Array ty -> Array ty -> Array ty
@@ -360,19 +361,19 @@ append a b = runST $ do
     unsafeCopyAtRO r (Offset 0) a (Offset 0) la
     unsafeCopyAtRO r (sizeAsOffset la) b (Offset 0) lb
     unsafeFreeze r
-  where la = lengthSize a
-        lb = lengthSize b
+  where la = length a
+        lb = length b
 
 concat :: [Array ty] -> Array ty
 concat l = runST $ do
-    r <- new (Size $ Prelude.sum $ fmap length l)
+    r <- new (mconcat $ fmap length l)
     loop r (Offset 0) l
     unsafeFreeze r
-  where loop _ _ []     = return ()
+  where loop _ _ []     = pure ()
         loop r i (x:xs) = do
             unsafeCopyAtRO r i x (Offset 0) lx
             loop r (i `offsetPlusE` lx) xs
-          where lx = lengthSize x
+          where lx = length x
 
 {-
 modify :: PrimMonad m
@@ -399,7 +400,7 @@ onNewArray len@(I# len#) f = primitive $ \st -> do
     case newArray# len# (error "onArray") st of { (# st2, mv #) ->
     case f mv st2                            of { st3           ->
     case unsafeFreezeArray# mv st3           of { (# st4, a #)  ->
-        (# st4, Array (Offset 0) (Size len) a #) }}}
+        (# st4, Array (Offset 0) (CountOf len) a #) }}}
 
 -----------------------------------------------------------------------
 
@@ -407,47 +408,53 @@ onNewArray len@(I# len#) f = primitive $ \st -> do
 null :: Array ty -> Bool
 null = (==) 0 . length
 
-take ::  Int -> Array ty -> Array ty
+take :: CountOf ty -> Array ty -> Array ty
 take nbElems a@(Array start len arr)
     | nbElems <= 0 = empty
     | n == len     = a
     | otherwise    = Array start n arr
   where
-    n = min (Size nbElems) len
+    n = min nbElems len
 
-drop ::  Int -> Array ty -> Array ty
+drop :: CountOf ty -> Array ty -> Array ty
 drop nbElems a@(Array start len arr)
     | nbElems <= 0 = a
     | n == len     = empty
     | otherwise    = Array (start `offsetPlusE` n) (len - n) arr
   where
-    n = min (Size nbElems) len
+    n = min nbElems len
 
-splitAt ::  Int -> Array ty -> (Array ty, Array ty)
+splitAt :: CountOf ty -> Array ty -> (Array ty, Array ty)
 splitAt nbElems a@(Array start len arr)
     | nbElems <= 0 = (empty, a)
     | n == len     = (a, empty)
     | otherwise    =
         (Array start n arr, Array (start `offsetPlusE` n) (len - n) arr)
   where
-    n = min (Size nbElems) len
+    n = min nbElems len
 
-revTake :: Int -> Array ty -> Array ty
-revTake nbElems v = drop (length v - nbElems) v
+-- inverse a CountOf that is specified from the end (e.g. take n elements from the end)
+countFromStart :: Array ty -> CountOf ty -> CountOf ty
+countFromStart v sz@(CountOf sz')
+    | sz >= len = CountOf 0
+    | otherwise = CountOf (len' - sz')
+  where len@(CountOf len') = length v
 
-revDrop :: Int -> Array ty -> Array ty
-revDrop nbElems v = take (length v - nbElems) v
+revTake :: CountOf ty -> Array ty -> Array ty
+revTake n v = drop (countFromStart v n) v
 
-revSplitAt :: Int -> Array ty -> (Array ty, Array ty)
-revSplitAt n v = (drop idx v, take idx v)
-  where idx = length v - n
+revDrop :: CountOf ty -> Array ty -> Array ty
+revDrop n v = take (countFromStart v n) v
+
+revSplitAt :: CountOf ty -> Array ty -> (Array ty, Array ty)
+revSplitAt n v = (drop idx v, take idx v) where idx = countFromStart v n
 
 splitOn ::  (ty -> Bool) -> Array ty -> [Array ty]
 splitOn predicate vec
-    | len == Size 0 = [mempty]
+    | len == CountOf 0 = [mempty]
     | otherwise     = loop (Offset 0) (Offset 0)
   where
-    !len = lengthSize vec
+    !len = length vec
     !endIdx = Offset 0 `offsetPlusE` len
     loop prevIdx idx
         | idx == endIdx = [sub vec prevIdx idx]
@@ -469,19 +476,19 @@ sub (Array start len a) startIdx expectedEndIdx
 break ::  (ty -> Bool) -> Array ty -> (Array ty, Array ty)
 break predicate v = findBreak 0
   where
-    !len = lengthSize v
-    findBreak i@(Offset i')
+    !len = length v
+    findBreak i
         | i .==# len  = (v, empty)
         | otherwise   =
             if predicate (unsafeIndex v i)
-                then splitAt i' v
+                then splitAt (offsetAsSize i) v
                 else findBreak (i+1)
 
 intersperse :: ty -> Array ty -> Array ty
 intersperse sep v
-    | len <= Size 1 = v
-    | otherwise     = runST $ unsafeCopyFrom v ((len + len) - Size 1) (go (Offset 0 `offsetPlusE` (len - Size 1)) sep)
-  where len = lengthSize v
+    | len <= CountOf 1 = v
+    | otherwise     = runST $ unsafeCopyFrom v ((len + len) - CountOf 1) (go (Offset 0 `offsetPlusE` (len - CountOf 1)) sep)
+  where len = length v
         -- terminate 1 before the end
 
         go :: Offset ty -> ty -> Array ty -> Offset ty -> MArray ty s -> ST s ()
@@ -498,7 +505,7 @@ span ::  (ty -> Bool) -> Array ty -> (Array ty, Array ty)
 span p = break (not . p)
 
 map :: (a -> b) -> Array a -> Array b
-map f a = create (sizeCast Proxy $ lengthSize a) (\i -> f $ unsafeIndex a (offsetCast Proxy i))
+map f a = create (sizeCast Proxy $ length a) (\i -> f $ unsafeIndex a (offsetCast Proxy i))
 
 {-
 mapIndex :: (Int -> a -> b) -> Array a -> Array b
@@ -511,19 +518,19 @@ singleton e = runST $ do
     unsafeWrite a 0 e
     unsafeFreeze a
 
-replicate :: Word -> ty -> Array ty
-replicate sz ty = create (Size (integralCast sz)) (const ty)
+replicate :: CountOf ty -> ty -> Array ty
+replicate sz ty = create sz (const ty)
 
 cons :: ty -> Array ty -> Array ty
 cons e vec
-    | len == Size 0 = singleton e
+    | len == CountOf 0 = singleton e
     | otherwise     = runST $ do
-        mv <- new (len + Size 1)
+        mv <- new (len + CountOf 1)
         unsafeWrite mv 0 e
         unsafeCopyAtRO mv (Offset 1) vec (Offset 0) len
         unsafeFreeze mv
   where
-    !len = lengthSize vec
+    !len = length vec
 
 snoc ::  Array ty -> ty -> Array ty
 snoc vec e
@@ -534,7 +541,7 @@ snoc vec e
         unsafeWrite mv (sizeAsOffset len) e
         unsafeFreeze mv
   where
-    !len = lengthSize vec
+    !len = length vec
 
 uncons :: Array ty -> Maybe (ty, Array ty)
 uncons vec
@@ -546,14 +553,23 @@ uncons vec
 unsnoc :: Array ty -> Maybe (Array ty, ty)
 unsnoc vec
     | len == 0  = Nothing
-    | otherwise = Just (take (lenI - 1) vec, unsafeIndex vec (sizeLastOffset len))
+    | otherwise = Just (take (len - 1) vec, unsafeIndex vec (sizeLastOffset len))
   where
-    !len@(Size lenI) = lengthSize vec
+    !len = length vec
 
-find ::  (ty -> Bool) -> Array ty -> Maybe ty
+elem :: Eq ty => ty -> Array ty -> Bool
+elem !ty arr = loop 0
+  where
+    !sz = length arr
+    loop !i | i .==# sz = False
+            | t == ty   = True
+            | otherwise = loop (i+1)
+      where t = unsafeIndex arr i
+
+find :: (ty -> Bool) -> Array ty -> Maybe ty
 find predicate vec = loop 0
   where
-    !len = lengthSize vec
+    !len = length vec
     loop i
         | i .==# len = Nothing
         | otherwise  =
@@ -565,12 +581,12 @@ sortBy xford vec
     | len == 0  = empty
     | otherwise = runST (thaw vec >>= doSort xford)
   where
-    len = lengthSize vec
+    len = length vec
     doSort :: PrimMonad prim => (ty -> ty -> Ordering) -> MArray ty (PrimState prim) -> prim (Array ty)
     doSort ford ma = qsort 0 (sizeLastOffset len) >> unsafeFreeze ma
       where
         qsort lo hi
-            | lo >= hi  = return ()
+            | lo >= hi  = pure ()
             | otherwise = do
                 p <- partition lo hi
                 qsort lo (pred p)
@@ -578,16 +594,16 @@ sortBy xford vec
         partition lo hi = do
             pivot <- unsafeRead ma hi
             let loop i j
-                    | j == hi   = return i
+                    | j == hi   = pure i
                     | otherwise = do
                         aj <- unsafeRead ma j
                         i' <- if ford aj pivot == GT
-                                then return i
+                                then pure i
                                 else do
                                     ai <- unsafeRead ma i
                                     unsafeWrite ma j ai
                                     unsafeWrite ma i aj
-                                    return $ i + 1
+                                    pure $ i + 1
                         loop i' (j+1)
 
             i <- loop lo lo
@@ -595,17 +611,17 @@ sortBy xford vec
             ahi <- unsafeRead ma hi
             unsafeWrite ma hi ai
             unsafeWrite ma i ahi
-            return i
+            pure i
 
 filter :: forall ty . (ty -> Bool) -> Array ty -> Array ty
 filter predicate vec = runST (new len >>= copyFilterFreeze predicate (unsafeIndex vec))
   where
-    !len = lengthSize vec
+    !len = length vec
     copyFilterFreeze :: PrimMonad prim => (ty -> Bool) -> (Offset ty -> ty) -> MArray ty (PrimState prim) -> prim (Array ty)
     copyFilterFreeze predi getVec mvec = loop (Offset 0) (Offset 0) >>= freezeUntilIndex mvec
       where
         loop d s
-            | s .==# len  = return d
+            | s .==# len  = pure d
             | predi v     = unsafeWrite mvec d v >> loop (d+1) (s+1)
             | otherwise   = loop d (s+1)
           where
@@ -617,27 +633,19 @@ freezeUntilIndex mvec d = do
     copyAt m (Offset 0) mvec (Offset 0) (offsetAsSize d)
     unsafeFreeze m
 
-unsafeFreezeShrink :: PrimMonad prim => MArray ty (PrimState prim) -> Size ty -> prim (Array ty)
+unsafeFreezeShrink :: PrimMonad prim => MArray ty (PrimState prim) -> CountOf ty -> prim (Array ty)
 unsafeFreezeShrink (MArray start _ ma) n = unsafeFreeze (MArray start n ma)
 
 reverse :: Array ty -> Array ty
 reverse a = create len toEnd
   where
-    len@(Size s) = lengthSize a
+    len@(CountOf s) = length a
     toEnd (Offset i) = unsafeIndex a (Offset (s - 1 - i))
 
-foldl :: (a -> ty -> a) -> a -> Array ty -> a
-foldl f initialAcc vec = loop 0 initialAcc
-  where
-    len = lengthSize vec
-    loop !i acc
-        | i .==# len = acc
-        | otherwise  = loop (i+1) (f acc (unsafeIndex vec i))
-
 foldr :: (ty -> a -> a) -> a -> Array ty -> a
 foldr f initialAcc vec = loop 0
   where
-    len = lengthSize vec
+    len = length vec
     loop !i
         | i .==# len = initialAcc
         | otherwise  = unsafeIndex vec i `f` loop (i+1)
@@ -645,41 +653,90 @@ foldr f initialAcc vec = loop 0
 foldl' :: (a -> ty -> a) -> a -> Array ty -> a
 foldl' f initialAcc vec = loop 0 initialAcc
   where
-    len = lengthSize vec
+    len = length vec
     loop !i !acc
         | i .==# len = acc
         | otherwise  = loop (i+1) (f acc (unsafeIndex vec i))
 
-builderAppend :: PrimMonad state => ty -> Builder (Array ty) (MArray ty) ty state ()
-builderAppend v = Builder $ State $ \(i, st) ->
+foldl1' :: (ty -> ty -> ty) -> NonEmpty (Array ty) -> ty
+foldl1' f arr = let (initialAcc, rest) = splitAt 1 $ getNonEmpty arr
+               in foldl' f (unsafeIndex initialAcc 0) rest
+
+foldr1 :: (ty -> ty -> ty) -> NonEmpty (Array ty) -> ty
+foldr1 f arr = let (initialAcc, rest) = revSplitAt 1 $ getNonEmpty arr
+               in foldr f (unsafeIndex initialAcc 0) rest
+
+all :: (ty -> Bool) -> Array ty -> Bool
+all p ba = loop 0
+  where
+    len = length ba
+    loop !i
+      | i .==# len = True
+      | not $ p (unsafeIndex ba i) = False
+      | otherwise = loop (i + 1)
+
+any :: (ty -> Bool) -> Array ty -> Bool
+any p ba = loop 0
+  where
+    len = length ba
+    loop !i
+      | i .==# len = False
+      | p (unsafeIndex ba i) = True
+      | otherwise = loop (i + 1)
+
+isPrefixOf :: Eq ty => Array ty -> Array ty -> Bool
+isPrefixOf pre arr
+    | pLen > pArr = False
+    | otherwise   = pre == take pLen arr
+  where
+    !pLen = length pre
+    !pArr = length arr
+
+isSuffixOf :: Eq ty => Array ty -> Array ty -> Bool
+isSuffixOf suffix arr
+    | pLen > pArr = False
+    | otherwise   = suffix == revTake pLen arr
+  where
+    !pLen = length suffix
+    !pArr = length arr
+
+builderAppend :: PrimMonad state => ty -> Builder (Array ty) (MArray ty) ty state err ()
+builderAppend v = Builder $ State $ \(i, st, e) ->
     if i .==# chunkSize st
         then do
             cur      <- unsafeFreeze (curChunk st)
             newChunk <- new (chunkSize st)
             unsafeWrite newChunk 0 v
-            return ((), (Offset 1, st { prevChunks     = cur : prevChunks st
+            pure ((), (Offset 1, st { prevChunks     = cur : prevChunks st
                                       , prevChunksSize = chunkSize st + prevChunksSize st
                                       , curChunk       = newChunk
-                                      }))
+                                      }, e))
         else do
             unsafeWrite (curChunk st) i v
-            return ((), (i+1, st))
+            pure ((), (i+1, st, e))
 
-builderBuild :: PrimMonad m => Int -> Builder (Array ty) (MArray ty) ty m () -> m (Array ty)
+builderBuild :: PrimMonad m => Int -> Builder (Array ty) (MArray ty) ty m err () -> m (Either err (Array ty))
 builderBuild sizeChunksI ab
     | sizeChunksI <= 0 = builderBuild 64 ab
     | otherwise        = do
         first         <- new sizeChunks
-        ((), (i, st)) <- runState (runBuilder ab) (Offset 0, BuildingState [] (Size 0) first sizeChunks)
-        cur           <- unsafeFreezeShrink (curChunk st) (offsetAsSize i)
-        -- Build final array
-        let totalSize = prevChunksSize st + offsetAsSize i
-        new totalSize >>= fillFromEnd totalSize (cur : prevChunks st) >>= unsafeFreeze
+        ((), (i, st, e)) <- runState (runBuilder ab) (Offset 0, BuildingState [] (CountOf 0) first sizeChunks, Nothing)
+        case e of
+          Just err -> pure (Left err)
+          Nothing -> do
+            cur <- unsafeFreezeShrink (curChunk st) (offsetAsSize i)
+            -- Build final array
+            let totalSize = prevChunksSize st + offsetAsSize i
+            bytes <- new totalSize >>= fillFromEnd totalSize (cur : prevChunks st) >>= unsafeFreeze
+            pure (Right bytes)
   where
-    sizeChunks = Size sizeChunksI
+    sizeChunks = CountOf sizeChunksI
 
-    fillFromEnd _   []     mua = return mua
+    fillFromEnd _   []     mua = pure mua
     fillFromEnd !end (x:xs) mua = do
-        let sz = lengthSize x
+        let sz = length x
         unsafeCopyAtRO mua (sizeAsOffset (end - sz)) x (Offset 0) sz
         fillFromEnd (end - sz) xs mua
+
+builderBuild_ :: PrimMonad m => Int -> Builder (Array ty) (MArray ty) ty m () () -> m (Array ty)
+builderBuild_ sizeChunksI ab = either (\() -> internalError "impossible output") id <$> builderBuild sizeChunksI ab
