@@ -15,6 +15,7 @@ module Foundation.Array.Internal
     ( UArray(..)
     , fromForeignPtr
     , withPtr
+    , copyToPtr
     , recast
     , toHexadecimal
     -- * Mutable facilities
@@ -24,4 +25,4 @@ module Foundation.Array.Internal
     ) where
 
 import           Foundation.Array.Unboxed
-import           Foundation.Array.Unboxed.Mutable
+import           Foundation.Array.Unboxed.Mutable hiding (copyToPtr)
