@@ -6,11 +6,12 @@ module Foundation.Boot.Builder
 
 import           Foundation.Internal.Base
 import           Foundation.Internal.MonadTrans
+import           Foundation.Monad.Exception
 import           Foundation.Primitive.Types.OffsetSize
 import           Foundation.Primitive.Monad
 
-newtype Builder collection mutCollection step state a = Builder
-    { runBuilder :: State (Offset step, BuildingState collection mutCollection step (PrimState state)) state a }
+newtype Builder collection mutCollection step state err a = Builder
+  { runBuilder :: State (Offset step, BuildingState collection mutCollection step (PrimState state), Maybe err) state a }
     deriving (Functor, Applicative, Monad)
 
 -- | The in-progress state of a building operation.
@@ -20,7 +21,12 @@ newtype Builder collection mutCollection step state a = Builder
 -- progress packing the elements inside.
 data BuildingState collection mutCollection step state = BuildingState
     { prevChunks     :: [collection]
-    , prevChunksSize :: !(Size step)
+    , prevChunksSize :: !(CountOf step)
     , curChunk       :: mutCollection state
-    , chunkSize      :: !(Size step)
+    , chunkSize      :: !(CountOf step)
     }
+
+instance Monad state => MonadFailure (Builder collection mutCollection step state err) where
+  type Failure (Builder collection mutCollection step state err) = err
+  mFail builderError = Builder $ State $ \(offset, bs, _)  ->
+    return ((), (offset, bs, Just builderError))
