@@ -9,7 +9,9 @@
 module Foundation.Internal.PrimTypes
     ( FileSize#
     , Offset#
-    , Size#
+    , CountOf#
+    , Bool#
+    , Pinned#
     ) where
 
 import GHC.Prim
@@ -22,7 +24,13 @@ type FileSize# = Word64#
 -- for code documentation purpose only, just a simple type alias on Int#
 type Offset# = Int#
 
--- | Size in bytes type alias
+-- | CountOf in bytes type alias
 --
 -- for code documentation purpose only, just a simple type alias on Int#
-type Size# = Int#
+type CountOf# = Int#
+
+-- | Lowlevel Boolean
+type Bool# = Int#
+
+-- | Pinning status
+type Pinned# = Bool#
