@@ -8,7 +8,6 @@
 -- Different collections (list, vector, string, ..) unified under 1 API.
 -- an API to rules them all, and in the darkness bind them.
 --
-{-# LANGUAGE DefaultSignatures #-}
 {-# LANGUAGE FlexibleInstances #-}
 module Foundation.Primitive
     ( PrimType(..)
@@ -28,6 +27,13 @@ module Foundation.Primitive
     , NormalForm(..)
     , force
     , deepseq
+
+    -- * These
+    , These(..)
+
+    -- * Block of memory
+    , Block
+    , MutableBlock
     ) where
 
 import Foundation.Primitive.Types
@@ -35,3 +41,5 @@ import Foundation.Primitive.Monad
 import Foundation.Primitive.Endianness
 import Foundation.Primitive.IntegralConv
 import Foundation.Primitive.NormalForm
+import Foundation.Primitive.These
+import Foundation.Primitive.Block
