@@ -11,6 +11,7 @@
 {-# LANGUAGE CPP #-}
 module Foundation.Primitive.Types
     ( PrimType(..)
+    , PrimMemoryComparable
     , primBaIndex
     , primMbaRead
     , primMbaWrite
@@ -22,6 +23,10 @@ module Foundation.Primitive.Types
     , sizeRecast
     , offsetAsSize
     , sizeAsOffset
+    , sizeInBytes
+    , offsetInBytes
+    , offsetInElements
+    , offsetIsAligned
     , primWordGetByteAndShift
     , primWord64GetByteAndShift
     , primWord64GetHiLo
@@ -33,9 +38,11 @@ import           GHC.Prim
 import           GHC.Int
 import           GHC.Types
 import           GHC.Word
+import           Data.Bits
 import           Foreign.C.Types
 import           Foundation.Internal.Proxy
 import           Foundation.Internal.Base
+import           Foundation.Numerical.Subtractive
 import           Foundation.Primitive.Types.OffsetSize
 import           Foundation.Primitive.Endianness
 import           Foundation.Primitive.Monad
@@ -51,7 +58,7 @@ divBytes :: PrimType ty => Offset ty -> (Int -> Int)
 divBytes ofs = \x -> x `Prelude.quot` (getSize Proxy ofs)
   where
     getSize :: PrimType ty => Proxy ty -> Offset ty -> Int
-    getSize p _ = let (Size sz) = primSizeInBytes p in sz
+    getSize p _ = let (CountOf sz) = primSizeInBytes p in sz
 
 baLength :: PrimType ty => Offset ty -> ByteArray# -> Int
 baLength ofs ba = divBytes ofs (I# (sizeofByteArray# ba))
@@ -153,6 +160,9 @@ class Eq ty => PrimType ty where
     -- | get the size in bytes of a ty element
     primSizeInBytes :: Proxy ty -> Size8
 
+    -- | get the shift size
+    primShiftToBytes :: Proxy ty -> Int
+
     -----
     -- ByteArray section
     -----
@@ -197,13 +207,18 @@ class Eq ty => PrimType ty where
                   -> ty
                   -> prim ()
 
-sizeInt, sizeWord :: Size Word8
+sizeInt, sizeWord :: CountOf Word8
+shiftInt, shiftWord :: Int
 #if WORD_SIZE_IN_BITS == 64
-sizeInt = Size 8
-sizeWord = Size 8
+sizeInt = CountOf 8
+sizeWord = CountOf 8
+shiftInt = 3
+shiftWord = 3
 #else
-sizeInt = Size 4
-sizeWord = Size 4
+sizeInt = CountOf 4
+sizeWord = CountOf 4
+shiftInt = 2
+shiftWord = 2
 #endif
 
 {-# SPECIALIZE [3] primBaUIndex :: ByteArray# -> Offset Word8 -> Word8 #-}
@@ -211,6 +226,8 @@ sizeWord = Size 4
 instance PrimType Int where
     primSizeInBytes _ = sizeInt
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = shiftInt
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = I# (indexIntArray# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readIntArray# mba n s1 in (# s2, I# r #)
@@ -227,6 +244,8 @@ instance PrimType Int where
 instance PrimType Word where
     primSizeInBytes _ = sizeWord
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = shiftWord
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = W# (indexWordArray# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readWordArray# mba n s1 in (# s2, W# r #)
@@ -241,8 +260,10 @@ instance PrimType Word where
     {-# INLINE primAddrWrite #-}
 
 instance PrimType Word8 where
-    primSizeInBytes _ = Size 1
+    primSizeInBytes _ = CountOf 1
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 0
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = W8# (indexWord8Array# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readWord8Array# mba n s1 in (# s2, W8# r #)
@@ -257,8 +278,10 @@ instance PrimType Word8 where
     {-# INLINE primAddrWrite #-}
 
 instance PrimType Word16 where
-    primSizeInBytes _ = Size 2
+    primSizeInBytes _ = CountOf 2
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 1
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = W16# (indexWord16Array# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readWord16Array# mba n s1 in (# s2, W16# r #)
@@ -272,8 +295,10 @@ instance PrimType Word16 where
     primAddrWrite addr (Offset (I# n)) (W16# w) = primitive $ \s1 -> (# writeWord16OffAddr# addr n w s1, () #)
     {-# INLINE primAddrWrite #-}
 instance PrimType Word32 where
-    primSizeInBytes _ = Size 4
+    primSizeInBytes _ = CountOf 4
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 2
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = W32# (indexWord32Array# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readWord32Array# mba n s1 in (# s2, W32# r #)
@@ -287,8 +312,10 @@ instance PrimType Word32 where
     primAddrWrite addr (Offset (I# n)) (W32# w) = primitive $ \s1 -> (# writeWord32OffAddr# addr n w s1, () #)
     {-# INLINE primAddrWrite #-}
 instance PrimType Word64 where
-    primSizeInBytes _ = Size 8
+    primSizeInBytes _ = CountOf 8
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 3
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = W64# (indexWord64Array# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readWord64Array# mba n s1 in (# s2, W64# r #)
@@ -302,8 +329,10 @@ instance PrimType Word64 where
     primAddrWrite addr (Offset (I# n)) (W64# w) = primitive $ \s1 -> (# writeWord64OffAddr# addr n w s1, () #)
     {-# INLINE primAddrWrite #-}
 instance PrimType Int8 where
-    primSizeInBytes _ = Size 1
+    primSizeInBytes _ = CountOf 1
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 0
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = I8# (indexInt8Array# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readInt8Array# mba n s1 in (# s2, I8# r #)
@@ -317,8 +346,10 @@ instance PrimType Int8 where
     primAddrWrite addr (Offset (I# n)) (I8# w) = primitive $ \s1 -> (# writeInt8OffAddr# addr n w s1, () #)
     {-# INLINE primAddrWrite #-}
 instance PrimType Int16 where
-    primSizeInBytes _ = Size 2
+    primSizeInBytes _ = CountOf 2
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 1
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = I16# (indexInt16Array# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readInt16Array# mba n s1 in (# s2, I16# r #)
@@ -332,8 +363,10 @@ instance PrimType Int16 where
     primAddrWrite addr (Offset (I# n)) (I16# w) = primitive $ \s1 -> (# writeInt16OffAddr# addr n w s1, () #)
     {-# INLINE primAddrWrite #-}
 instance PrimType Int32 where
-    primSizeInBytes _ = Size 4
+    primSizeInBytes _ = CountOf 4
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 2
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = I32# (indexInt32Array# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readInt32Array# mba n s1 in (# s2, I32# r #)
@@ -347,8 +380,10 @@ instance PrimType Int32 where
     primAddrWrite addr (Offset (I# n)) (I32# w) = primitive $ \s1 -> (# writeInt32OffAddr# addr n w s1, () #)
     {-# INLINE primAddrWrite #-}
 instance PrimType Int64 where
-    primSizeInBytes _ = Size 8
+    primSizeInBytes _ = CountOf 8
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 3
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = I64# (indexInt64Array# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readInt64Array# mba n s1 in (# s2, I64# r #)
@@ -363,8 +398,10 @@ instance PrimType Int64 where
     {-# INLINE primAddrWrite #-}
 
 instance PrimType Float where
-    primSizeInBytes _ = Size 4
+    primSizeInBytes _ = CountOf 4
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 2
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = F# (indexFloatArray# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readFloatArray# mba n s1 in (# s2, F# r #)
@@ -378,8 +415,10 @@ instance PrimType Float where
     primAddrWrite addr (Offset (I# n)) (F# w) = primitive $ \s1 -> (# writeFloatOffAddr# addr n w s1, () #)
     {-# INLINE primAddrWrite #-}
 instance PrimType Double where
-    primSizeInBytes _ = Size 8
+    primSizeInBytes _ = CountOf 8
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 3
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = D# (indexDoubleArray# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readDoubleArray# mba n s1 in (# s2, D# r #)
@@ -394,8 +433,10 @@ instance PrimType Double where
     {-# INLINE primAddrWrite #-}
 
 instance PrimType Char where
-    primSizeInBytes _ = Size 4
+    primSizeInBytes _ = CountOf 4
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 2
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset (I# n)) = C# (indexWideCharArray# ba n)
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset (I# n)) = primitive $ \s1 -> let (# s2, r #) = readWideCharArray# mba n s1 in (# s2, C# r #)
@@ -410,8 +451,10 @@ instance PrimType Char where
     {-# INLINE primAddrWrite #-}
 
 instance PrimType CChar where
-    primSizeInBytes _ = Size 1
+    primSizeInBytes _ = CountOf 1
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 0
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset n) = CChar (primBaUIndex ba (Offset n))
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset n) = CChar <$> primMbaURead mba (Offset n)
@@ -425,8 +468,10 @@ instance PrimType CChar where
     primAddrWrite addr (Offset n) (CChar int8) = primAddrWrite addr (Offset n) int8
     {-# INLINE primAddrWrite #-}
 instance PrimType CUChar where
-    primSizeInBytes _ = Size 1
+    primSizeInBytes _ = CountOf 1
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = 0
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset n) = CUChar (primBaUIndex ba (Offset n :: Offset Word8))
     {-# INLINE primBaUIndex #-}
     primMbaURead mba (Offset n) = CUChar <$> primMbaURead mba (Offset n :: Offset Word8)
@@ -443,6 +488,8 @@ instance PrimType CUChar where
 instance PrimType a => PrimType (LE a) where
     primSizeInBytes _ = primSizeInBytes (Proxy :: Proxy a)
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = primShiftToBytes (Proxy :: Proxy a)
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset a) = LE $ primBaUIndex ba (Offset a)
     {-# INLINE primBaUIndex #-}
     primMbaURead ba (Offset a) = LE <$> primMbaURead ba (Offset a)
@@ -458,6 +505,8 @@ instance PrimType a => PrimType (LE a) where
 instance PrimType a => PrimType (BE a) where
     primSizeInBytes _ = primSizeInBytes (Proxy :: Proxy a)
     {-# INLINE primSizeInBytes #-}
+    primShiftToBytes _ = primShiftToBytes (Proxy :: Proxy a)
+    {-# INLINE primShiftToBytes #-}
     primBaUIndex ba (Offset a) = BE $ primBaUIndex ba (Offset a)
     {-# INLINE primBaUIndex #-}
     primMbaURead ba (Offset a) = BE <$> primMbaURead ba (Offset a)
@@ -471,38 +520,90 @@ instance PrimType a => PrimType (BE a) where
     primAddrWrite addr (Offset a) (BE w) = primAddrWrite addr (Offset a) w
     {-# INLINE primAddrWrite #-}
 
+-- | A constraint class for serializable type that have an unique
+-- memory compare representation
+--
+-- e.g. Float and Double have -0.0 and 0.0 which are Eq individual,
+-- yet have a different memory representation which doesn't allow
+-- for memcmp operation
+class PrimMemoryComparable ty where
 
--- | Cast a Size linked to type A (Size A) to a Size linked to type B (Size B)
-sizeRecast :: (PrimType a, PrimType b) => Size a -> Size b
-sizeRecast = doRecast Proxy Proxy
-  where doRecast :: (PrimType a, PrimType b) => Proxy a -> Proxy b -> Size a -> Size b
-        doRecast pa pb sz =
-            let szA          = primSizeInBytes pa
-                (Size szB)   = primSizeInBytes pb
-                (Size bytes) = sizeOfE szA sz
-             in Size (bytes `Prelude.quot` szB)
+instance PrimMemoryComparable Int where
+instance PrimMemoryComparable Word where
+instance PrimMemoryComparable Word8 where
+instance PrimMemoryComparable Word16 where
+instance PrimMemoryComparable Word32 where
+instance PrimMemoryComparable Word64 where
+instance PrimMemoryComparable Int8 where
+instance PrimMemoryComparable Int16 where
+instance PrimMemoryComparable Int32 where
+instance PrimMemoryComparable Int64 where
+instance PrimMemoryComparable Char where
+instance PrimMemoryComparable CChar where
+instance PrimMemoryComparable CUChar where
+instance PrimMemoryComparable a => PrimMemoryComparable (LE a) where
+instance PrimMemoryComparable a => PrimMemoryComparable (BE a) where
 
-primOffsetRecast :: (PrimType a, PrimType b) => Offset a -> Offset b
-primOffsetRecast = doRecast Proxy Proxy
-  where doRecast :: (PrimType a, PrimType b) => Proxy a -> Proxy b -> Offset a -> Offset b
-        doRecast pa pb ofs =
-            let szA            = primSizeInBytes pa
-                (Size szB)     = primSizeInBytes pb
-                (Offset bytes) = offsetOfE szA ofs
-             in Offset (bytes `Prelude.quot` szB)
+-- | Cast a CountOf linked to type A (CountOf A) to a CountOf linked to type B (CountOf B)
+sizeRecast :: forall a b . (PrimType a, PrimType b) => CountOf a -> CountOf b
+sizeRecast sz = CountOf (bytes `Prelude.quot` szB)
+  where !szA             = primSizeInBytes (Proxy :: Proxy a)
+        !(CountOf szB)   = primSizeInBytes (Proxy :: Proxy b)
+        !(CountOf bytes) = sizeOfE szA sz
+{-# INLINE [1] sizeRecast #-}
+{-# RULES "sizeRecast from Word8" [2] forall a . sizeRecast a = sizeRecastBytes a #-}
 
-primOffsetOfE :: PrimType a => Offset a -> Offset8
-primOffsetOfE = getOffset Proxy
-  where getOffset :: PrimType a => Proxy a -> Offset a -> Offset8
-        getOffset proxy = offsetOfE (primSizeInBytes proxy)
+sizeRecastBytes :: forall b . PrimType b => CountOf Word8 -> CountOf b
+sizeRecastBytes (CountOf w) = CountOf (w `Prelude.quot` szB)
+  where !(CountOf szB) = primSizeInBytes (Proxy :: Proxy b)
+{-# INLINE [1] sizeRecastBytes #-}
 
-sizeAsOffset :: Size a -> Offset a
-sizeAsOffset (Size a) = Offset a
-{-# INLINE sizeAsOffset #-}
+sizeInBytes :: forall a . PrimType a => CountOf a -> CountOf Word8
+sizeInBytes sz = sizeOfE (primSizeInBytes (Proxy :: Proxy a)) sz
 
-offsetAsSize :: Offset a -> Size a
-offsetAsSize (Offset a) = Size a
-{-# INLINE offsetAsSize #-}
+offsetInBytes :: forall a . PrimType a => Offset a -> Offset Word8
+offsetInBytes ofs = offsetShiftL (primShiftToBytes (Proxy :: Proxy a)) ofs
+{-# INLINE [2] offsetInBytes #-}
+{-# SPECIALIZE INLINE [3] offsetInBytes :: Offset Word64 -> Offset Word8 #-}
+{-# SPECIALIZE INLINE [3] offsetInBytes :: Offset Word32 -> Offset Word8 #-}
+{-# SPECIALIZE INLINE [3] offsetInBytes :: Offset Word16 -> Offset Word8 #-}
+{-# RULES "offsetInBytes Bytes" [3] forall x . offsetInBytes x = x #-}
+
+offsetInElements :: forall a . PrimType a => Offset Word8 -> Offset a
+offsetInElements ofs = offsetShiftR (primShiftToBytes (Proxy :: Proxy a)) ofs
+{-# INLINE [2] offsetInElements #-}
+{-# SPECIALIZE INLINE [3] offsetInBytes :: Offset Word64 -> Offset Word8 #-}
+{-# SPECIALIZE INLINE [3] offsetInBytes :: Offset Word32 -> Offset Word8 #-}
+{-# SPECIALIZE INLINE [3] offsetInBytes :: Offset Word16 -> Offset Word8 #-}
+{-# RULES "offsetInElements Bytes" [3] forall x . offsetInElements x = x #-}
+
+primOffsetRecast :: forall a b . (PrimType a, PrimType b) => Offset a -> Offset b
+primOffsetRecast !ofs =
+    let !(Offset bytes) = offsetOfE szA ofs
+     in Offset (bytes `Prelude.quot` szB)
+  where
+    !szA        = primSizeInBytes (Proxy :: Proxy a)
+    !(CountOf szB) = primSizeInBytes (Proxy :: Proxy b)
+{-# INLINE [1] primOffsetRecast #-}
+{-# RULES "primOffsetRecast W8" [3] forall a . primOffsetRecast a = primOffsetRecastBytes a #-}
+
+offsetIsAligned :: forall a . PrimType a => Proxy a -> Offset Word8 -> Bool
+offsetIsAligned _ (Offset ofs) = (ofs .&. mask) == 0
+   where (CountOf sz) = primSizeInBytes (Proxy :: Proxy a)
+         mask = sz - 1
+{-# INLINE [1] offsetIsAligned #-}
+{-# SPECIALIZE [3] offsetIsAligned :: Proxy Word64 -> Offset Word8 -> Bool #-}
+{-# RULES "offsetInAligned Bytes" [3] forall (prx :: Proxy Word8) x . offsetIsAligned prx x = True #-}
+
+primOffsetRecastBytes :: forall b . PrimType b => Offset Word8 -> Offset b
+primOffsetRecastBytes (Offset 0) = Offset 0
+primOffsetRecastBytes (Offset o) = Offset (szA `Prelude.quot` o)
+  where !(CountOf szA) = primSizeInBytes (Proxy :: Proxy b)
+{-# INLINE [1] primOffsetRecastBytes #-}
+
+primOffsetOfE :: forall a . PrimType a => Offset a -> Offset Word8
+primOffsetOfE = offsetInBytes
+{-# DEPRECATED primOffsetOfE "use offsetInBytes" #-}
 
 primWordGetByteAndShift :: Word# -> (# Word#, Word# #)
 primWordGetByteAndShift w = (# and# w 0xff##, uncheckedShiftRL# w 8# #)
