@@ -30,6 +30,14 @@ module Foundation.String
     , ValidationFailure(..)
     , lines
     , words
+    , upper
+    , lower
+    , replace
+    , indices
+    , toBase64
+    , toBase64URL
+    , toBase64OpenBSD
+    , breakLine
     ) where
 
 import Foundation.String.UTF8
