@@ -21,8 +21,8 @@ createPtr :: forall e . StorableFixed e => [e] -> IO (FinalPtr e)
 createPtr l
     | null l    = toFinalPtr nullPtr (\_ -> return ())
     | otherwise = do
-        let (Size szElem) = size (Proxy :: Proxy e)
-            nbBytes = szElem * length l
+        let (CountOf szElem) = size (Proxy :: Proxy e)
+            nbBytes = szElem * (let (CountOf c) = length l in c)
         ptr <- mallocBytes nbBytes
-        forM_ (zip [0..] l) $ \(o, e) -> pokeOff ptr o e
-        toFinalPtr ptr (\p -> free p)
+        forM_ (zip [0..] l) $ uncurry (pokeOff ptr)
+        toFinalPtr ptr free
